import { CmcdObjectType } from '@svta/common-media-library/cmcd/CmcdObjectType';
import { CmcdStreamType } from '@svta/common-media-library/cmcd/CmcdStreamType';
import { CmcdStreamingFormat } from '@svta/common-media-library/cmcd/CmcdStreamingFormat';
export var CmcdData = /*#__PURE__*/function () {
  function CmcdData(player, sid, cid) {
    this.player = player;
    this.vhs = player.tech(true).vhs;
    this.sid = sid;
    this.cid = cid;
  }

  var _proto = CmcdData.prototype;

  _proto.getEncodedBitrate = function getEncodedBitrate() {
    try {
      var bandwidth = this.player.tech(true).vhs.playlists.media_.attributes.BANDWIDTH;
      var encodedBitrate = Math.round(bandwidth / 1000);
      return encodedBitrate;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getObjectDuration = function getObjectDuration(uriBeingRequested) {
    try {
      var playlist = this.player.tech(true).vhs.playlists.media();
      var currentSegmentIndex = playlist.segments.findIndex(function (segment) {
        return segment.resolvedUri === uriBeingRequested;
      });
      var currentSegmentDuration = Math.round(playlist.segments[currentSegmentIndex].duration * 1000);
      return currentSegmentDuration;
    } catch (error) {
      return undefined;
    }
  };

  _proto.getObjectType = function getObjectType(uriBeingRequested) {
    try {
      var extension = uriBeingRequested.split('.').pop();
      var MANIFEST = CmcdObjectType.MANIFEST,
          AUDIO = CmcdObjectType.AUDIO,
          VIDEO = CmcdObjectType.VIDEO,
          MUXED = CmcdObjectType.MUXED,
          CAPTION = CmcdObjectType.CAPTION,
          OTHER = CmcdObjectType.OTHER;
      var supportedExtensions = {
        // Manifest or playlist
        m3u8: MANIFEST,
        mpd: MANIFEST,
        xml: MANIFEST,
        // Audio Only
        m4a: AUDIO,
        amp3ac: AUDIO,
        aac: AUDIO,
        caf: AUDIO,
        flac: AUDIO,
        oga: AUDIO,
        wav: AUDIO,
        // Video Only
        opus: VIDEO,
        ogv: VIDEO,
        mp4: VIDEO,
        mov: VIDEO,
        m4v: VIDEO,
        mkv: VIDEO,
        webm: VIDEO,
        ogg: VIDEO,
        flv: VIDEO,
        // Muxed audio and video
        ts: MUXED,
        // Init segment not implemented
        // Caption
        webvtt: CAPTION,
        vtt: CAPTION,
        // ISOBMFF timed text track not implemented
        // Cryptographic key, license or certificate not implemented
        // Other
        jpg: OTHER,
        jpeg: OTHER,
        gif: OTHER,
        png: OTHER,
        svg: OTHER,
        webp: OTHER
      };

      if (supportedExtensions.hasOwnProperty(extension)) {
        return supportedExtensions[extension];
      }

      return undefined;
    } catch (error) {
      return undefined;
    }
  };

  _proto.getTopBitrate = function getTopBitrate() {
    try {
      var qualitylevels = this.player.qualityLevels().levels_;
      var highestBitrate = qualitylevels.reduce(function (prev, current) {
        return prev && prev.bitrate > current.bitrate ? prev : current;
      });
      var topBitrate = Math.round(highestBitrate.bitrate / 1000);
      return topBitrate;
    } catch (error) {
      return undefined;
    }
  };

  _proto.bufferLengthMs = function bufferLengthMs() {
    try {
      var bufferLength = 0;
      var tech = this.player.tech(true);
      var buffered = tech.buffered();

      if (buffered.length > 0) {
        var lastBufferedTime = buffered.end(buffered.length - 1);

        if (!isNaN(lastBufferedTime)) {
          bufferLength = lastBufferedTime - tech.currentTime();
        }
      }

      var bufferLengthMs = bufferLength * 1000;
      return bufferLengthMs;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getBufferLength = function getBufferLength() {
    // This key SHOULD only be sent with an object type of ‘a’, ‘v’ or ‘av’.
    try {
      var bufferLengthMs = this.bufferLengthMs();
      return bufferLengthMs;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getDeadline = function getDeadline() {
    try {
      var bufferLength = this.bufferLengthMs();
      var playbackRate = this.player.playbackRate() * 1000;
      var deadline = Math.round(bufferLength / playbackRate);
      return deadline;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getMeasuredThroughput = function getMeasuredThroughput() {
    try {
      var bandwidth = Math.round(this.vhs.systemBandwidth / 1000);
      return bandwidth;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getNextObjectRequest = function getNextObjectRequest(actualURIrequest) {
    try {
      var nextObject;

      if (this.player.duration().toString() !== 'Infinity' && this.player.duration() !== 0) {
        // This logic doesn't work when is live video
        var segments = this.vhs.playlists.media().segments;
        var segmentIndexFind = segments.findIndex(function (seg) {
          return seg.resolvedUri === actualURIrequest;
        });

        if (segmentIndexFind !== -1 && segmentIndexFind !== segments.length) {
          nextObject = segments[segmentIndexFind + 1].uri;
        }
      }

      return nextObject;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getNextRangeRequest = function getNextRangeRequest() {
    // TODO
    return undefined;
  };

  _proto.generateHashCode = function generateHashCode(string) {
    var hash = 0;

    if (string.length === 0) {
      return hash;
    }

    for (var i = 0; i < string.length; i++) {
      var chr = string.charCodeAt(i);
      hash = (hash << 5) - hash + chr;
      hash |= 0;
    }

    return hash;
  };

  _proto.getContentId = function getContentId(src) {
    if (this.cid) {
      return this.cid;
    }

    try {
      var cid = this.generateHashCode(src);
      return cid.toString();
    } catch (e) {
      return undefined;
    }
  };

  _proto.getPlaybackRate = function getPlaybackRate() {
    try {
      var rate = this.player.playbackRate();

      if (rate === 1) {
        return 1;
      } else if (rate === 0) {
        return 0;
      }

      return 2;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getStreamingFormat = function getStreamingFormat() {
    try {
      var type = this.player.currentType();

      if (type === 'application/dash+xml') {
        return CmcdStreamingFormat.DASH;
      } else if (type === 'application/x-mpegURL') {
        return CmcdStreamingFormat.HLS;
      } else if (type === 'application/vnd.ms-sstr+xml') {
        return CmcdStreamingFormat.SMOOTH;
      }

      return CmcdStreamingFormat.OTHER;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getStreamType = function getStreamType() {
    try {
      // It is a live video
      if (this.player.duration().toString() === 'Infinity' || this.player.duration() === 0) {
        return CmcdStreamType.LIVE;
      } // else it is a vod video


      return CmcdStreamType.VOD;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getVersion = function getVersion() {
    return 1;
  };

  _proto.getBufferStarvation = function getBufferStarvation(isWaitingEvent) {
    try {
      return isWaitingEvent;
    } catch (e) {
      return undefined;
    }
  };

  _proto.getRequestedMaximumThroughput = function getRequestedMaximumThroughput() {
    // TODO
    return undefined;
  };

  _proto.getKeys = function getKeys(uri, isWaitingEvent, src) {
    var res = {
      br: this.getEncodedBitrate(),
      d: this.getObjectDuration(uri),
      ot: this.getObjectType(uri),
      tb: this.getTopBitrate()
    };

    if ([CmcdObjectType.AUDIO, CmcdObjectType.VIDEO, CmcdObjectType.MUXED].includes(res.ot)) {
      res.bl = this.getBufferLength();
    }

    res.dl = this.getDeadline();
    res.mtp = this.getMeasuredThroughput();
    res.nor = this.getNextObjectRequest(uri);
    res.nrr = this.getNextRangeRequest();

    if (isWaitingEvent !== false) {
      res.su = isWaitingEvent;
    }

    res.cid = this.getContentId(src);

    if (this.getPlaybackRate() !== 1) {
      res.pr = this.getPlaybackRate();
    }

    res.sf = this.getStreamingFormat();
    res.sid = this.sid;
    res.st = this.getStreamType();

    if (this.getVersion() !== 1) {
      res.v = this.getVersion();
    }

    if (isWaitingEvent !== false) {
      res.bs = this.getBufferStarvation(isWaitingEvent);
    }

    res.rtp = this.getRequestedMaximumThroughput();
    return res;
  };

  return CmcdData;
}();