import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import { appendCmcdHeaders } from '@svta/common-media-library/cmcd/appendCmcdHeaders';
import { appendCmcdQuery } from '@svta/common-media-library/cmcd/appendCmcdQuery';
import { uuid } from '@svta/common-media-library/utils/uuid';
import videojs from 'video.js';
import { version as VERSION } from '../package.json';
import { CmcdData } from './CmcdData.js';
var Plugin = videojs.getPlugin('plugin');
var isWaitingEvent = true; // Default options for the plugin.

var defaults = {};
/**
 * An advanced Video.js plugin. For more information on the API
 *
 * See: https://blog.videojs.com/feature-spotlight-advanced-plugins/
 */

var Cmcd = /*#__PURE__*/function (_Plugin) {
  _inheritsLoose(Cmcd, _Plugin);

  /**
   * Create a Cmcd plugin instance.
   *
   * @param  {Player} player
   *         A Video.js Player instance.
   *
   * @param  {Object} [options]
   *         An optional options object.
   *
   *         While not a core part of the Video.js plugin architecture, a
   *         second argument of options is a convenient way to accept inputs
   *         from your plugin's caller.
   */
  function Cmcd(player, options) {
    var _this;

    // the parent class will add player under this.player
    _this = _Plugin.call(this, player) || this;
    _this.options = videojs.obj.merge(defaults, options);

    var _ref = options || {},
        sid = _ref.sid,
        cid = _ref.cid,
        useHeaders = _ref.useHeaders;

    _this.cid = cid;
    _this.sid = sid || uuid();
    _this.useHeaders = useHeaders;

    _this.player.ready(function () {
      player.addClass('vjs-cmcd');
      handleEvents(player);
      player.on('xhr-hooks-ready', function () {
        var playerXhrRequestHook = function playerXhrRequestHook(opts) {
          var cmcd = new CmcdData(_this.player, _this.sid, _this.cid);
          var keys = cmcd.getKeys(opts.uri, isWaitingEvent, _this.player.currentSrc());

          if (_this.useHeaders) {
            var headers = appendCmcdHeaders({}, keys);

            opts.beforeSend = function (xhr) {
              for (var h in headers) {
                xhr.setRequestHeader(h, headers[h]);
              }
            };
          } else {
            opts.uri = appendCmcdQuery(opts.uri, keys);
          }

          return opts;
        };

        player.tech().vhs.xhr.onRequest(playerXhrRequestHook);
      });
    });

    return _this;
  }

  var _proto = Cmcd.prototype;

  _proto.setId = function setId(id) {
    if (id.sid) {
      this.sid = id.sid;
    }

    if (id.cid) {
      this.cid = id.cid;
    }
  };

  return Cmcd;
}(Plugin);

function handleEvents(player) {
  // startup
  player.on('loadedmetadata', function () {
    isWaitingEvent = false;
  }); // seeking or buffer-empty event

  player.on('waiting', function () {
    isWaitingEvent = true;
  }); // all it's okey

  player.on('canplay', function () {
    isWaitingEvent = false;
  });
} // Define default values for the plugin's `state` object here.


Cmcd.defaultState = {}; // Include the version number.

Cmcd.VERSION = VERSION; // Register the plugin with video.js.

videojs.registerPlugin('cmcd', Cmcd);
export default Cmcd;